@@ -1,4 +1,4 @@
-.multispatialccm_ts_method = \(data, cause, effect, libsizes, E = 3, tau = 0, k = E+1, boot = 99, seed = 42, dist.metric = "L1",
+.multispatialccm_ts_method = \(data, cause, effect, libsizes, E = 3, tau = 1, k = E+1, lib = NULL, boot = 99, seed = 42, dist.metric = "L1",
                                dist.average = TRUE,threads = length(libsizes),parallel.level = "low",bidirectional = TRUE,progressbar = TRUE){
   varname = .check_character(cause,effect)
   E = .check_inputelementnum(E,2)
@@ -7,12 +7,13 @@
   pl = .check_parallellevel(parallel.level)
   cause = as.matrix(data[[cause]])
   effect = as.matrix(data[[effect]])
+  if (is.null(lib)) lib = seq_len(ncol(cause))
 
   x_xmap_y = NULL
   if (bidirectional){
-    x_xmap_y = RcppMultispatialCCM(cause,effect,libsizes,E[1],tau[1],k[1],boot,seed,threads,pl,.check_distmetric(dist.metric),dist.average,progressbar)
+    x_xmap_y = RcppMultispatialCCM(cause,effect,libsizes,lib,E[1],tau[1],k[1],boot,seed,threads,pl,.check_distmetric(dist.metric),dist.average,progressbar)
   }
-  y_xmap_x = RcppMultispatialCCM(effect,cause,libsizes,E[2],tau[2],k[2],boot,seed,threads,pl,.check_distmetric(dist.metric),dist.average,progressbar)
+  y_xmap_x = RcppMultispatialCCM(effect,cause,libsizes,lib,E[2],tau[2],k[2],boot,seed,threads,pl,.check_distmetric(dist.metric),dist.average,progressbar)
 
   return(.bind_xmapdf(varname,x_xmap_y,y_xmap_x,bidirectional))
 }
@@ -26,6 +27,7 @@
 #' @param E (optional) embedding dimensions.
 #' @param tau (optional) step of time lags.
 #' @param k (optional) number of nearest neighbors used in prediction.
+#' @param lib (optional) libraries indices.
 #' @param boot (optional) number of bootstraps to perform.
 #' @param seed (optional) random seed.
 #' @param dist.metric (optional) distance metric (`L1`: Manhattan, `L2`: Euclidean).
@@ -38,7 +40,7 @@
 #' @return A list
 #' \describe{
 #' \item{\code{xmap}}{cross mapping results}
-#' \item{\code{varname}}{names of causal and effect variable}
+#' \item{\code{varname}}{names of causal and effect variables}
 #' \item{\code{bidirectional}}{whether to examine bidirectional causality}
 #' }
 #' @export
@@ -49,13 +51,13 @@
 #'
 #' @examples
 #' set.seed(42)
-#' obs = runif(15,0,0.1)
-#' sim = vector("list",15)
+#' obs = runif(10,0,0.1)
+#' sim = vector("list",5)
 #' for (i in seq_along(obs)){
 #'   sim[[i]] = logistic_map(x = obs[i],y = obs[i],step = 15,beta_xy = 0.5,beta_yx = 0)
 #' }
 #' lst = list(x = do.call(cbind, lapply(sim, function(df) df$x)),
 #'            y = do.call(cbind, lapply(sim, function(df) df$y)))
-#' multispatialccm(lst,"x","y",libsizes = 5:15,E = c(7,2),k = 6,threads = 1)
+#' multispatialccm(lst,"x","y",libsizes = 1:5,E = 2,k = 3,threads = 1)
 #'
 methods::setMethod("multispatialccm", "list", .multispatialccm_ts_method)
