@@ -11,8 +11,6 @@
 #include "SMap.h"
 #include <RcppThread.h>
 
-// [[Rcpp::depends(RcppThread)]]
-
 /*
  * Perform convergent cross mapping on a single lib and pred.
  *
@@ -156,8 +154,9 @@ std::vector<std::pair<int, double>> CCMSingle(
  * - theta: Distance weighting parameter used for weighting neighbors in the S-mapping prediction.
  * - threads: Number of threads to use for parallel computation.
  * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
- *   dist_metric    - Distance metric selector (1: Manhattan, 2: Euclidean).
- *   dist_average   - Whether to average distance by the number of valid vector components.
+ * - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * - dist_average: Whether to average distance by the number of valid vector components.
+ * - single_sig: Whether to estimate significance and confidence intervals using a single rho value.
  * - progressbar: Boolean flag to indicate whether to display a progress bar during computation.
  *
  * Returns:
@@ -165,8 +164,8 @@ std::vector<std::pair<int, double>> CCMSingle(
  *      - The library size.
  *      - The mean cross-mapping correlation.
  *      - The statistical significance of the correlation.
- *      - The upper bound of the confidence interval.
  *      - The lower bound of the confidence interval.
+ *      - The upper bound of the confidence interval.
  */
 std::vector<std::vector<double>> CCM(
     const std::vector<double>& x,
@@ -183,6 +182,7 @@ std::vector<std::vector<double>> CCM(
     int parallel_level = 0,
     int dist_metric = 2,
     bool dist_average = true,
+    bool single_sig = true,
     bool progressbar = false
 ) {
   // If b is not provided correctly, default it to E + 1
@@ -310,45 +310,45 @@ std::vector<std::vector<double>> CCM(
     grouped_results[result.first].push_back(result.second);
   }
 
-  // // Previous implementation calculated significance and confidence intervals using the mean of rho vector only.
-  // // This approach is now deprecated and kept here for comparison purposes.
-  // std::vector<std::vector<double>> final_results;
-  // for (const auto& group : grouped_results) {
-  //   double mean_value = CppMean(group.second, true);
-  //   final_results.push_back({static_cast<double>(group.first), mean_value});
-  // }
-  //
-  // // Calculate significance and confidence interval for each result
-  // for (size_t i = 0; i < final_results.size(); ++i) {
-  //   double rho = final_results[i][1];
-  //   double significance = CppCorSignificance(rho, n);
-  //   std::vector<double> confidence_interval = CppCorConfidence(rho, n);
-  //
-  //   final_results[i].push_back(significance);
-  //   final_results[i].push_back(confidence_interval[0]);
-  //   final_results[i].push_back(confidence_interval[1]);
-  // }
-
-  // Refactor correlation analysis to compute significance and confidence intervals directly from grouped correlation vectors
   std::vector<std::vector<double>> final_results;
-  for (const auto& group : grouped_results) {
-    // Calculate the mean correlation coefficient from the group
-    double mean_value = CppMean(group.second, true);
+  if (single_sig) {
+    // Calculate significance and confidence intervals using the mean of rho vector only.
+    for (const auto& group : grouped_results) {
+      double mean_value = CppMean(group.second, true);
+      final_results.push_back({static_cast<double>(group.first), mean_value});
+    }
 
-    // Compute significance (p-value) using the vector of correlations directly
-    double significance = CppMeanCorSignificance(group.second, n);
+    // Calculate significance and confidence interval for each result
+    for (size_t i = 0; i < final_results.size(); ++i) {
+      double rho = final_results[i][1];
+      double significance = CppCorSignificance(rho, n);
+      std::vector<double> confidence_interval = CppCorConfidence(rho, n);
 
-    // Compute confidence interval using the vector of correlations directly
-    std::vector<double> confidence_interval = CppMeanCorConfidence(group.second, n);
+      final_results[i].push_back(significance);
+      final_results[i].push_back(confidence_interval[0]);
+      final_results[i].push_back(confidence_interval[1]);
+    }
+  } else {
+    // Compute significance and confidence intervals directly from grouped correlation vectors
+    for (const auto& group : grouped_results) {
+      // Calculate the mean correlation coefficient from the group
+      double mean_value = CppMean(group.second, true);
 
-    // Store results: group ID, mean correlation, p-value, lower CI, upper CI
-    final_results.push_back({
-      static_cast<double>(group.first),
-      mean_value,
-      significance,
-      confidence_interval[0],
-      confidence_interval[1]
-    });
+      // Compute significance (p-value) using the vector of correlations directly
+      double significance = CppMeanCorSignificance(group.second, n);
+
+      // Compute confidence interval using the vector of correlations directly
+      std::vector<double> confidence_interval = CppMeanCorConfidence(group.second, n);
+
+      // Store results: group ID, mean correlation, p-value, lower CI, upper CI
+      final_results.push_back({
+        static_cast<double>(group.first),
+        mean_value,
+        significance,
+        confidence_interval[0],
+        confidence_interval[1]
+      });
+    }
   }
 
   return final_results;
