@@ -12,7 +12,12 @@
 #include "Embed.h"
 #include "SimplexProjection.h"
 #include "SMap.h"
-#include <RcppThread.h>
+
+// Note: <RcppThread.h> is intentionally excluded from this header to avoid
+//       unnecessary Rcpp dependencies and potential header inclusion order
+//       issues (e.g., R.h being included before Rcpp headers). It should only
+//       be included in the corresponding .cpp implementation file.
+// #include <RcppThread.h>
 
 /*
  * Perform convergent cross mapping on a single lib and pred.
@@ -67,8 +72,9 @@ std::vector<std::pair<int, double>> CCMSingle(
  * - theta: Distance weighting parameter used for weighting neighbors in the S-mapping prediction.
  * - threads: Number of threads to use for parallel computation.
  * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
- *   dist_metric    - Distance metric selector (1: Manhattan, 2: Euclidean).
- *   dist_average   - Whether to average distance by the number of valid vector components.
+ * - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
+ * - dist_average: Whether to average distance by the number of valid vector components.
+ * - single_sig: Whether to estimate significance and confidence intervals using a single rho value.
  * - progressbar: Boolean flag to indicate whether to display a progress bar during computation.
  *
  * Returns:
@@ -76,8 +82,8 @@ std::vector<std::pair<int, double>> CCMSingle(
  *      - The library size.
  *      - The mean cross-mapping correlation.
  *      - The statistical significance of the correlation.
- *      - The upper bound of the confidence interval.
  *      - The lower bound of the confidence interval.
+ *      - The upper bound of the confidence interval.
  */
 std::vector<std::vector<double>> CCM(
     const std::vector<double>& x,
@@ -94,6 +100,7 @@ std::vector<std::vector<double>> CCM(
     int parallel_level = 0,
     int dist_metric = 2,
     bool dist_average = true,
+    bool single_sig = true,
     bool progressbar = false
 );
 
