@@ -8,7 +8,6 @@
 #include "DeLongPlacements.h"
 // #include <Rcpp.h>
 #include <RcppArmadillo.h>
-// [[Rcpp::depends(RcppArmadillo)]]
 
 // Function to check if a value is NA
 bool isNA(double value) {
@@ -32,13 +31,13 @@ bool checkOneDimVectorHasNaN(const std::vector<double>& vec) {
 
 // Function to count the number of non-NaN values in a one-dimensional vector
 int checkOneDimVectorNotNanNum(const std::vector<double>& vec) {
-  int count = 0; // Initialize the counter for non-NaN values
+  size_t count = 0; // Initialize the counter for non-NaN values
   for (double val : vec) {
     if (!std::isnan(val)) {
       ++count; // Increment the counter if the value is not NaN
     }
   }
-  return count; // Return the count of non-NaN values
+  return static_cast<int>(count); // Return the count of non-NaN values
 }
 
 /**
@@ -136,6 +135,9 @@ double CppLog(double x, double base = 10) {
 // Function to calculate the median of a vector.
 double CppMedian(const std::vector<double>& vec, bool NA_rm = false) {
   std::vector<double> filtered_vec;
+  if (filtered_vec.capacity() < vec.size()){
+    filtered_vec.reserve(vec.size());
+  }
   for (const double& value : vec) {
     if (std::isnan(value)) {
       if (!NA_rm) {
@@ -202,7 +204,6 @@ double CppMin(const std::vector<double>& vec, bool NA_rm = false) {
   return found_valid ? min_val : std::numeric_limits<double>::quiet_NaN();
 }
 
-
 // Function to calculate the maximum of a vector, ignoring NA values if NA_rm is true
 double CppMax(const std::vector<double>& vec, bool NA_rm = false) {
   double max_val = -std::numeric_limits<double>::infinity();
@@ -393,6 +394,85 @@ std::vector<double> CppArithmeticSeq(double from, double to, size_t length_out)
   return res;
 }
 
+/**
+ * @brief Compute quantiles of a numeric vector at specified probabilities.
+ *
+ * This function calculates the quantiles of a given vector of doubles `vec` at
+ * user-defined probability levels specified in `probs` (defaulting to 0.05, 0.5, 0.95).
+ * The quantiles are computed using linear interpolation between sorted values.
+ *
+ * Optionally, the function can remove NaN values before calculation if `NA_rm` is true,
+ * ensuring robust quantile estimates in the presence of missing data.
+ *
+ * @param vec     Input vector of double values.
+ * @param probs   Vector of probabilities in [0,1] at which to compute quantiles.
+ *                Default is {0.05, 0.5, 0.95}.
+ * @param NA_rm   Whether to remove NaN values from `vec` before quantile calculation.
+ *                Default is true.
+ *
+ * @return A vector of doubles containing quantile values corresponding to `probs`.
+ *
+ * @note If `NA_rm` is false and `vec` contains NaNs, the result may be undefined or contain NaNs.
+ *       It is recommended to set `NA_rm` to true when input may have missing values.
+ */
+std::vector<double> CppQuantile(const std::vector<double>& vec,
+                                const std::vector<double>& probs = {0.05, 0.5, 0.95},
+                                bool NA_rm = true) {
+  // Filter input vector if NA_rm == true (remove NaNs)
+  std::vector<double> clean_vec;
+  if (NA_rm) {
+    clean_vec.reserve(vec.size());
+    for (auto v : vec) {
+      if (!std::isnan(v)) {
+        clean_vec.push_back(v);
+      }
+    }
+  } else {
+    clean_vec = vec;
+  }
+
+  if (clean_vec.empty()) {
+    // If no data after removing NaNs, return vector of NaNs of same size as probs
+    return std::vector<double>(probs.size(), std::numeric_limits<double>::quiet_NaN());
+  }
+
+  // Sort the data
+  std::sort(clean_vec.begin(), clean_vec.end());
+
+  std::vector<double> results;
+  results.reserve(probs.size());
+
+  // Compute quantiles using linear interpolation (Type 7 method like R's quantile default)
+  // h = 1 + (n - 1) * p
+  // quantile = (1 - gamma) * x[j] + gamma * x[j+1]
+  size_t n = clean_vec.size();
+
+  for (double p : probs) {
+    if (p < 0.0 || p > 1.0) {
+      throw std::out_of_range("probabilities must be between 0 and 1");
+    }
+
+    if (n == 1) {
+      // Only one element
+      results.push_back(clean_vec[0]);
+      continue;
+    }
+
+    double h = 1 + (n - 1) * p;
+    size_t h_floor = static_cast<size_t>(std::floor(h));
+    double gamma = h - h_floor;
+
+    // Indices adjusted for 0-based indexing in C++
+    size_t idx_lower = (h_floor == 0) ? 0 : h_floor - 1;
+    size_t idx_upper = (h_floor >= n) ? n - 1 : h_floor;
+
+    double q = (1 - gamma) * clean_vec[idx_lower] + gamma * clean_vec[idx_upper];
+    results.push_back(q);
+  }
+
+  return results;
+}
+
 // Function to calculate the variance of a vector, ignoring NA values
 double CppVariance(const std::vector<double>& vec, bool NA_rm = false) {
   double mean_val = CppMean(vec, NA_rm);
@@ -439,8 +519,11 @@ double PearsonCor(const std::vector<double>& y,
 
   // Handle NA values
   std::vector<double> clean_y, clean_y_hat;
+  clean_y.reserve(y.size());
+  clean_y_hat.reserve(y_hat.size());
+
   for (size_t i = 0; i < y.size(); ++i) {
-    bool is_na = isNA(y[i]) || isNA(y_hat[i]);
+    bool is_na = std::isnan(y[i]) || std::isnan(y_hat[i]);
     if (is_na) {
       if (!NA_rm) {
         return std::numeric_limits<double>::quiet_NaN(); // Return NaN if NA_rm is false
@@ -456,9 +539,13 @@ double PearsonCor(const std::vector<double>& y,
     return std::numeric_limits<double>::quiet_NaN();
   }
 
-  // Convert cleaned vectors to Armadillo vectors
-  arma::vec arma_y(clean_y);
-  arma::vec arma_y_hat(clean_y_hat);
+  // // Convert cleaned vectors to Armadillo vectors
+  // arma::vec arma_y(clean_y);
+  // arma::vec arma_y_hat(clean_y_hat);
+
+  // Safe zero-copy view (valid while clean_y is alive)
+  arma::vec arma_y(clean_y.data(), clean_y.size(), false);
+  arma::vec arma_y_hat(clean_y_hat.data(), clean_y_hat.size(), false);
 
   // Compute Pearson correlation using Armadillo
   double corr = arma::as_scalar(arma::cor(arma_y, arma_y_hat));
@@ -531,8 +618,11 @@ double SpearmanCor(const std::vector<double>& y,
   }
 
   std::vector<double> clean_y, clean_y_hat;
+  clean_y.reserve(y.size());
+  clean_y_hat.reserve(y_hat.size());
+
   for (size_t i = 0; i < y.size(); ++i) {
-    bool is_na = isNA(y[i]) || isNA(y_hat[i]);
+    bool is_na = std::isnan(y[i]) || std::isnan(y_hat[i]);
     if (is_na) {
       if (!NA_rm) {
         return std::numeric_limits<double>::quiet_NaN();
@@ -547,8 +637,9 @@ double SpearmanCor(const std::vector<double>& y,
     return std::numeric_limits<double>::quiet_NaN();
   }
 
-  arma::vec arma_y(clean_y);
-  arma::vec arma_y_hat(clean_y_hat);
+  // Safe zero-copy view (valid while clean_y is alive)
+  arma::vec arma_y(clean_y.data(), clean_y.size(), false);
+  arma::vec arma_y_hat(clean_y_hat.data(), clean_y_hat.size(), false);
 
   // Rank-transform both vectors
   arma::vec rank_y = arma::conv_to<arma::vec>::from(arma::sort_index(arma::sort_index(arma_y))) + 1;
@@ -568,6 +659,9 @@ double KendallCor(const std::vector<double>& y,
   }
 
   std::vector<double> clean_y, clean_y_hat;
+  clean_y.reserve(y.size());
+  clean_y_hat.reserve(y_hat.size());
+
   for (size_t i = 0; i < y.size(); ++i) {
     bool is_na = isNA(y[i]) || isNA(y_hat[i]);
     if (is_na) {
@@ -612,14 +706,14 @@ double KendallCor(const std::vector<double>& y,
  * Optionally, missing values (NA) can be removed if 'NA_rm' is set to true.
  *
  * Parameters:
- *   y            - A vector representing the dependent variable.
- *   y_hat        - A vector representing the predicted variable.
- *   controls     - A matrix where each row corresponds to a control variable to adjust for in the correlation.
- *   NA_rm        - A boolean flag to indicate whether to remove missing values (default is false).
- *   linear       - A boolean flag to specify whether to use linear regression (true) or correlation matrix (false)
- *                  for computing the partial correlation (default is false).
- *   pinv_tol     - Tolerance used for the pseudo-inverse (arma::pinv). Smaller values increase precision but may be less stable
- *                  (default is 1e-10).
+ *   y          - A vector representing the dependent variable.
+ *   y_hat      - A vector representing the predicted variable.
+ *   controls   - A matrix where each row corresponds to a control variable to adjust for in the correlation.
+ *   NA_rm      - A boolean flag to indicate whether to remove missing values (default is false).
+ *   linear     - A boolean flag to specify whether to use linear regression (true) or correlation matrix (false)
+ *                for computing the partial correlation (default is false).
+ *   pinv_tol   - Tolerance used for the pseudo-inverse (arma::pinv). Smaller values increase precision but may be less stable
+ *                (default is 1e-10).
  *
  * Returns:
  *   A double representing the partial correlation coefficient between 'y' and 'y_hat' after controlling for
@@ -645,7 +739,10 @@ double PartialCor(const std::vector<double>& y,
 
   // Handle NA values
   std::vector<double> clean_y, clean_y_hat;
+  clean_y.reserve(y.size());
+  clean_y_hat.reserve(y_hat.size());
   std::vector<std::vector<double>> clean_controls(controls.size());
+  for (auto& c : clean_controls) c.reserve(y.size());
 
   for (size_t i = 0; i < y.size(); ++i) {
     bool is_na = isNA(y[i]) || isNA(y_hat[i]);
@@ -693,8 +790,9 @@ double PartialCor(const std::vector<double>& y,
     // arma::vec residuals_y_hat = arma_y_hat - arma_controls * arma::solve(arma_controls, arma_y_hat);
 
     // Use a more robust method for solving the linear system, such as arma::pinv (pseudo-inverse):
-    arma::vec residuals_y = arma_y - arma_controls * arma::pinv(arma_controls) * arma_y;
-    arma::vec residuals_y_hat = arma_y_hat - arma_controls * arma::pinv(arma_controls) * arma_y_hat;
+    arma::mat pinv_controls = arma::pinv(arma_controls);
+    arma::vec residuals_y = arma_y - arma_controls * pinv_controls * arma_y;
+    arma::vec residuals_y_hat = arma_y_hat - arma_controls * pinv_controls * arma_y_hat;
 
     // Compute Pearson correlation of the residuals
     partial_corr = arma::as_scalar(arma::cor(residuals_y, residuals_y_hat));
@@ -703,11 +801,23 @@ double PartialCor(const std::vector<double>& y,
     int i = controls.size();
     int j = controls.size() + 1;
     arma::mat data(clean_y.size(), i + 2);
+
+    // for (size_t k = 0; k < controls.size(); ++k) {
+    //   data.col(k) = arma::vec(clean_controls[k]);
+    // }
+    // data.col(i) = arma::vec(clean_y);
+    // data.col(j) = arma::vec(clean_y_hat);
+
     for (size_t k = 0; k < controls.size(); ++k) {
-      data.col(k) = arma::vec(clean_controls[k]);
+      arma::vec view(clean_controls[k].data(),
+                     clean_controls[k].size(),
+                     false);
+      data.col(k) = view;
     }
-    data.col(i) = arma::vec(clean_y);
-    data.col(j) = arma::vec(clean_y_hat);
+    arma::vec y_view(clean_y.data(), clean_y.size(), false);
+    arma::vec yhat_view(clean_y_hat.data(), clean_y_hat.size(), false);
+    data.col(i) = y_view;
+    data.col(j) = yhat_view;
 
     if (data.n_rows < 2 || data.n_cols < 1) {
       return std::numeric_limits<double>::quiet_NaN();
@@ -716,11 +826,6 @@ double PartialCor(const std::vector<double>& y,
     // Compute the correlation matrix of the data
     arma::mat corrm = arma::cor(data);
 
-    // Check matrix validity
-    if (!corrm.is_finite()) {
-      return std::numeric_limits<double>::quiet_NaN();
-    }
-
     // // Compute the precision matrix (inverse of the correlation matrix)
     // arma::mat precm = arma::inv(corrm);
 
@@ -790,7 +895,7 @@ double CppCorSignificance(double r, size_t n, size_t k = 0) {
 
   double pvalue = (1 - R::pt(t, df, true, false)) * 2;
 
-  // Ensure p value is within valid range [-1, 1]
+  // Ensure p value is within valid range [0, 1]
   if (pvalue < 0) pvalue = 0;
   if (pvalue > 1.0) pvalue = 1.0;
 
@@ -817,7 +922,7 @@ double CppCorSignificance(double r, size_t n, size_t k = 0) {
  * @param n The number of observations.
  * @param k The number of control variables (default = 0; use 0 for simple correlation).
  * @param level The significance level α for the confidence interval (default = 0.05).
- * @return A vector containing the upper and lower bounds of the confidence interval.
+ * @return A vector containing the lower and upper bounds of the confidence interval.
  */
 std::vector<double> CppCorConfidence(double r, size_t n, size_t k = 0,
                                      double level = 0.05) {
@@ -850,7 +955,7 @@ std::vector<double> CppCorConfidence(double r, size_t n, size_t k = 0,
   if (r_upper > 1.0) r_upper = 1.0;
 
   // Return the result as a std::vector<double>
-  return {r_upper, r_lower};
+  return {r_lower, r_upper};
 }
 
 /**
@@ -861,14 +966,15 @@ std::vector<double> CppCorConfidence(double r, size_t n, size_t k = 0,
  * - If only one valid correlation coefficient is provided, it uses a t-distribution
  *   to compute the p-value.
  *
- * - If multiple valid correlation coefficients are provided (e.g., from bootstrapping),
- *   it applies Fisher's z-transformation, computes the mean z, and evaluates significance
- *   using the standard normal distribution.
+ * - If multiple valid correlation coefficients are provided (e.g., from bootstrapping
+ *   or repeated windowing), it applies Fisher's z-transformation, computes the mean z,
+ *   and evaluates significance using the standard normal distribution, adjusting the
+ *   standard error by sqrt(m).
  *
  * Correlation coefficients must be within the valid range (-1, 1); NaN values are ignored.
  *
  * @param rho_vec A vector of correlation or partial correlation coefficients.
- * @param n The number of observations.
+ * @param n The number of observations in each correlation estimate.
  * @param k The number of control variables (0 for simple correlation, >0 for partial correlation).
  * @return The two-sided p-value; returns NaN if no valid rho is found.
  */
@@ -888,26 +994,25 @@ double CppMeanCorSignificance(const std::vector<double>& rho_vec,
 
   if (m == 1) {
     // Use t-statistic for single correlation
-    double r = rho_vec[0];
+    double r = z_values[0];  // use z_values to avoid invalid rho
+    r = std::tanh(r);        // back-transform
     double df = static_cast<double>(n - k - 2);
     double t = r * std::sqrt(df / (1.0 - r * r));
     double pvalue = 2.0 * R::pt(-std::fabs(t), df, true, false);
 
-    // Clamp p-value to [0,1] to prevent numerical issues
     if (pvalue < 0.0) pvalue = 0.0;
     if (pvalue > 1.0) pvalue = 1.0;
 
     return pvalue;
   }
 
-  // Use Fisher z transformation for multiple correlations
+  // Multiple correlations: Fisher z + mean
   double z_mean = std::accumulate(z_values.begin(), z_values.end(), 0.0) / m;
-  double se = 1.0 / std::sqrt(static_cast<double>(n - k - 3));
+  double se = 1.0 / std::sqrt(static_cast<double>(m) * (n - k - 3));
   double z_stat = z_mean / se;
 
   double pvalue = 2.0 * R::pnorm(-std::fabs(z_stat), 0.0, 1.0, true, false);
 
-  // Clamp p-value to [0,1]
   if (pvalue < 0.0) pvalue = 0.0;
   if (pvalue > 1.0) pvalue = 1.0;
 
@@ -922,14 +1027,15 @@ double CppMeanCorSignificance(const std::vector<double>& rho_vec,
  * - If only one valid correlation coefficient is provided, it uses Fisher's z-transformation
  *   to compute the confidence interval using the standard normal quantile.
  *
- * - If multiple valid correlation coefficients are provided (e.g., from bootstrapping),
- *   it applies Fisher's z-transformation to each, averages them, and computes the confidence interval
- *   on the transformed scale, then transforms back to correlation scale.
+ * - If multiple valid correlation coefficients are provided (e.g., from bootstrapping
+ *   or repeated windowing), it applies Fisher's z-transformation to each, averages them,
+ *   and computes the confidence interval on the transformed scale, adjusting the standard
+ *   error by sqrt(m), then transforms back to correlation scale.
  *
  * Correlation coefficients must be within the valid range (-1, 1); NaN values are ignored.
  *
  * @param rho_vec A vector of correlation or partial correlation coefficients.
- * @param n The number of observations.
+ * @param n The number of observations in each correlation estimate.
  * @param k The number of control variables (0 for simple correlation).
  * @param level The significance level alpha (default = 0.05 for 95% confidence).
  * @return A vector of two elements: lower and upper bounds of the confidence interval;
@@ -939,7 +1045,6 @@ std::vector<double> CppMeanCorConfidence(const std::vector<double>& rho_vec,
                                          size_t n, size_t k = 0,
                                          double level = 0.05) {
   if (n <= k + 3) {
-    // Not enough degrees of freedom
     return {std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN()};
   }
 
@@ -954,7 +1059,7 @@ std::vector<double> CppMeanCorConfidence(const std::vector<double>& rho_vec,
   if (m == 0) return {std::numeric_limits<double>::quiet_NaN(), std::numeric_limits<double>::quiet_NaN()};
 
   double z_mean = std::accumulate(z_values.begin(), z_values.end(), 0.0) / m;
-  double se = 1.0 / std::sqrt(static_cast<double>(n - k - 3));
+  double se = 1.0 / std::sqrt(static_cast<double>(m) * (n - k - 3));
   double z_crit = R::qnorm(1.0 - level / 2.0, 0.0, 1.0, true, false);
 
   double z_lower = z_mean - z_crit * se;
@@ -963,7 +1068,6 @@ std::vector<double> CppMeanCorConfidence(const std::vector<double>& rho_vec,
   double r_lower = (std::exp(2 * z_lower) - 1) / (std::exp(2 * z_lower) + 1);
   double r_upper = (std::exp(2 * z_upper) - 1) / (std::exp(2 * z_upper) + 1);
 
-  // Clamp results to [-1,1] to prevent numerical issues
   if (r_lower < -1.0) r_lower = -1.0;
   if (r_lower > 1.0) r_lower = 1.0;
   if (r_upper < -1.0) r_upper = -1.0;
@@ -1030,7 +1134,7 @@ std::vector<double> CppDeLongAUCConfidence(const std::vector<double>& cases,
   ci_upper = std::min(1.0, ci_upper);
 
   // Return the results as a three-element vector
-  return {theta, ci_upper, ci_lower};
+  return {theta, ci_lower, ci_upper};
 }
 
 /**
@@ -1109,7 +1213,7 @@ std::vector<double> CppCMCTest(const std::vector<double>& cases,
   ci_upper = std::min(1.0, ci_upper);
 
   // Return the results as a four-element vector
-  return {theta, p_value, ci_upper, ci_lower};
+  return {theta, p_value, ci_lower, ci_upper};
 }
 
 // /**
@@ -1123,8 +1227,8 @@ std::vector<double> CppCMCTest(const std::vector<double>& cases,
 //  * @return A vector of four elements:
 //  *   - area_auc: AUC computed by trapezoidal rule (area method).
 //  *   - p_value: The p-value testing H0: AUC = 0.5 (via DeLong).
-//  *   - ci_upper: Upper bound of 100*(1-level)% confidence interval (via DeLong).
 //  *   - ci_lower: Lower bound of 100*(1-level)% confidence interval (via DeLong).
+//  *   - ci_upper: Upper bound of 100*(1-level)% confidence interval (via DeLong).
 //  */
 // std::vector<double> CppCMCTest(const std::vector<double>& cases,
 //                                const std::string& direction,
@@ -1201,7 +1305,7 @@ std::vector<double> CppCMCTest(const std::vector<double>& cases,
 //   ci_lower = std::max(0.0, ci_lower);
 //   ci_upper = std::min(1.0, ci_upper);
 //
-//   return {area_auc, p_value, ci_upper, ci_lower};
+//   return {area_auc, p_value, ci_lower, ci_upper};
 // }
 
 /*
