@@ -9,8 +9,6 @@
 #include "SimplexProjection.h"
 #include <RcppThread.h>
 
-// [[Rcpp::depends(RcppThread)]]
-
 /*
  * Perform bootstrapped simplex prediction from spatially replicated short time series.
  * Supports parallel execution using RcppThread if parallel_level == 0.
@@ -18,11 +16,12 @@
  * Parameters:
  *   source         - A vector of vectors representing explanatory variables (plots × time).
  *   target         - A vector of vectors representing response variables (plots × time).
+ *   libsize        - Number of plots to sample in each bootstrap replicate.
+ *   lib_indices    - Vector of indices indicating which plots to include when searching for neighbors.
  *   E              - Embedding dimension.
  *   tau            - Time delay.
- *   lib            - Number of plots to sample in each bootstrap replicate.
  *   num_neighbors  - Number of nearest neighbors to use in simplex projection.
- *   threads        - Number of nearest neighbors to use in bootstrap replicates.
+ *   threads        - Number of paralleled threads to use in bootstrap replicates.
  *   seed           - Random seed for reproducibility.
  *   boot           - Number of bootstrap replicates.
  *   parallel_level - If 0, run in parallel using RcppThread. Otherwise, run sequentially.
@@ -34,42 +33,30 @@
  *     [0] = library size (libsize)
  *     [1] = mean of Pearson correlation (mean rho)
  *     [2] = one-sided p-value (proportion of rho ≤ 0)
- *     [3] = 97.5% percentile (upper bound of 95% CI)
- *     [4] = 2.5% percentile (lower bound of 95% CI)
+ *     [3] = 2.5% percentile (lower bound of 95% CI)
+ *     [4] = 97.5% percentile (upper bound of 95% CI)
  */
 std::vector<double> SimplexPredictionBoot(
     const std::vector<std::vector<double>>& source,
     const std::vector<std::vector<double>>& target,
-    int E,
-    int tau,
-    int lib,
-    int num_neighbors,
-    int boot,
-    size_t threads,
+    int libsize,
+    const std::vector<int>& lib_indices,
+    int E = 3,
+    int tau = 1,
+    int num_neighbors = 4,
+    int boot = 1,
+    size_t threads = 8,
     unsigned int seed = 42,
     int parallel_level = 0,
     int dist_metric = 2,
     bool dist_average = true
 ) {
-  int n_plot = source.size();
-  std::vector<double> rho_list(boot, std::numeric_limits<double>::quiet_NaN());
+  if (boot == 1) {
+    std::vector<double> res(5, std::numeric_limits<double>::quiet_NaN());
+    res[0] = static_cast<double>(libsize);
 
-  // Prebuild RNG pool with seed sequence
-  std::vector<std::mt19937> rng_pool(boot);
-  for (int i = 0; i < boot; ++i) {
-    std::seed_seq seq{static_cast<uint32_t>(seed), static_cast<uint32_t>(i)};
-    rng_pool[i] = std::mt19937(seq);
-  }
-
-  auto run_boot_once = [&](int b) {
-    std::mt19937& rng = rng_pool[b];
-    std::uniform_int_distribution<> plot_sampler(0, n_plot - 1);
-
-    // 1. Bootstrap plots
-    std::vector<int> lib_plots(lib);
-    for (int i = 0; i < lib; ++i) {
-      lib_plots[i] = plot_sampler(rng);
-    }
+    // 1. Construct plots
+    std::vector<int> lib_plots(lib_indices.begin(), lib_indices.begin() + libsize);
 
     // 2. Embedding
     std::vector<std::vector<double>> library_vectors;
@@ -80,16 +67,16 @@ std::vector<double> SimplexPredictionBoot(
       int T = target[plot].size();
       for (int i = 0; i < static_cast<int>(emb.size()); ++i) {
         int ti = i + tau * (E - 1);
-        if (ti + 1 < T) {
-          library_vectors.push_back(emb[i]);
-          library_targets.push_back(target[plot][ti + 1]);
+        if (ti < T) {  // only require that Y(t) exists
+          library_vectors.push_back(emb[ti]);
+          library_targets.push_back(target[plot][ti]);  // ← predict Y(t), not Y(t+1)
         }
       }
     }
 
     int N = library_vectors.size();
     if (N < num_neighbors) {
-      return;
+      return res;
     }
 
     std::vector<int> all_indices(N);
@@ -99,42 +86,95 @@ std::vector<double> SimplexPredictionBoot(
                                             all_indices, all_indices, num_neighbors,
                                             dist_metric, dist_average);
 
-    rho_list[b] = PearsonCor(library_targets, pred, true);
-  };
+    res[1] = PearsonCor(library_targets, pred, true);
 
-  // Parallel or sequential execution
-  if (parallel_level == 0) {
-    RcppThread::parallelFor(0, boot, run_boot_once, threads);
+    return res;
   } else {
-    for (int b = 0; b < boot; ++b) run_boot_once(b);
-  }
+    int n_plot = lib_indices.size();
+    std::vector<double> rho_list(boot, std::numeric_limits<double>::quiet_NaN());
 
-  // 3. Summary statistics
-  double mean_rho = CppMean(rho_list, true);
-
-  int n_non_na = 0, count_le_0 = 0;
-  std::vector<double> clean_rho;
-  for (double r : rho_list) {
-    if (!std::isnan(r)) {
-      ++n_non_na;
-      clean_rho.push_back(r);
-      if (r <= 0) ++count_le_0;
+    // Prebuild 64-bit RNG pool with seed sequence
+    std::vector<std::mt19937_64> rng_pool(boot);
+    for (int i = 0; i < boot; ++i) {
+      std::seed_seq seq{static_cast<uint64_t>(seed), static_cast<uint64_t>(i)};
+      rng_pool[i] = std::mt19937_64(seq);
     }
+
+    auto run_boot_once = [&](int b) {
+      std::mt19937_64& rng = rng_pool[b];
+      std::uniform_int_distribution<> plot_sampler(0, n_plot - 1);
+
+      // 1. Bootstrap plots
+      std::vector<int> lib_plots(libsize);
+      for (int i = 0; i < libsize; ++i) {
+        lib_plots[i] = lib_indices[plot_sampler(rng)];
+      }
+
+      // 2. Embedding
+      std::vector<std::vector<double>> library_vectors;
+      std::vector<double> library_targets;
+
+      for (int plot : lib_plots) {
+        auto emb = Embed(source[plot], E, tau);
+        int T = target[plot].size();
+        for (int i = 0; i < static_cast<int>(emb.size()); ++i) {
+          int ti = i + tau * (E - 1);
+          if (ti < T) {  // only require that Y(t) exists
+            library_vectors.push_back(emb[ti]);
+            library_targets.push_back(target[plot][ti]);  // ← predict Y(t), not Y(t+1)
+          }
+        }
+      }
+
+      int N = library_vectors.size();
+      if (N < num_neighbors) {
+        return;
+      }
+
+      std::vector<int> all_indices(N);
+      std::iota(all_indices.begin(), all_indices.end(), 0);
+
+      auto pred = SimplexProjectionPrediction(library_vectors, library_targets,
+                                              all_indices, all_indices, num_neighbors,
+                                              dist_metric, dist_average);
+
+      rho_list[b] = PearsonCor(library_targets, pred, true);
+    };
+
+    // Parallel or sequential execution
+    if (parallel_level == 0) {
+      RcppThread::parallelFor(0, boot, run_boot_once, threads);
+    } else {
+      for (int b = 0; b < boot; ++b) run_boot_once(b);
+    }
+
+    // 3. Summary statistics
+    double mean_rho = CppMean(rho_list, true);
+
+    int n_non_na = 0, count_le_0 = 0;
+    std::vector<double> clean_rho;
+    for (double r : rho_list) {
+      if (!std::isnan(r)) {
+        ++n_non_na;
+        clean_rho.push_back(r);
+        if (r <= 0) ++count_le_0;
+      }
+    }
+
+    double pval = (n_non_na > 0) ? (1.0 + count_le_0) / (1.0 + n_non_na)
+      : std::numeric_limits<double>::quiet_NaN();
+
+    std::sort(clean_rho.begin(), clean_rho.end());
+    double ci_lower = std::numeric_limits<double>::quiet_NaN();
+    double ci_upper = std::numeric_limits<double>::quiet_NaN();
+    int n = clean_rho.size();
+    if (n >= 1) {
+      ci_lower = clean_rho[std::clamp(int(std::floor(0.025 * n)), 0, n - 1)];
+      ci_upper = clean_rho[std::clamp(int(std::ceil(0.975 * n)) - 1, 0, n - 1)];
+    }
+
+    return {static_cast<double>(libsize), mean_rho, pval, ci_lower, ci_upper};
   }
-
-  double pval = (n_non_na > 0) ? (1.0 + count_le_0) / (1.0 + n_non_na)
-    : std::numeric_limits<double>::quiet_NaN();
-
-  std::sort(clean_rho.begin(), clean_rho.end());
-  double ci_lower = std::numeric_limits<double>::quiet_NaN();
-  double ci_upper = std::numeric_limits<double>::quiet_NaN();
-  int n = clean_rho.size();
-  if (n >= 1) {
-    ci_lower = clean_rho[std::clamp(int(std::floor(0.025 * n)), 0, n - 1)];
-    ci_upper = clean_rho[std::clamp(int(std::ceil(0.975 * n)) - 1, 0, n - 1)];
-  }
-
-  return {static_cast<double>(lib), mean_rho, pval, ci_upper, ci_lower};
 }
 
 /*
@@ -153,6 +193,7 @@ std::vector<double> SimplexPredictionBoot(
  *   x              - A vector of vectors representing explanatory variables (plots × time).
  *   y              - A vector of vectors representing response variables (plots × time).
  *   lib_sizes      - A list of library sizes to evaluate (number of plots to sample).
+ *   lib            - A vector of representing the indices of sample plots to be the library.
  *   E              - Embedding dimension for state space reconstruction.
  *   tau            - Time delay between lags in the embedding.
  *   b              - Number of nearest neighbors used in simplex projection (defaults to E + 1).
@@ -169,18 +210,19 @@ std::vector<double> SimplexPredictionBoot(
  *     [0] = library size (libsize)
  *     [1] = mean of Pearson correlation (mean rho)
  *     [2] = one-sided p-value (proportion of rho ≤ 0 across bootstraps)
- *     [3] = 97.5% percentile of rho distribution (upper CI bound)
- *     [4] = 2.5% percentile of rho distribution (lower CI bound)
+ *     [3] = 2.5% percentile of rho distribution (lower CI bound)
+ *     [4] = 97.5% percentile of rho distribution (upper CI bound)
  */
 std::vector<std::vector<double>> MultispatialCCM(
     const std::vector<std::vector<double>>& x,
     const std::vector<std::vector<double>>& y,
     const std::vector<int>& lib_sizes,
-    int E,
-    int tau,
-    int b,
-    int boot,
-    int threads,
+    const std::vector<int>& lib,
+    int E = 3,
+    int tau = 1,
+    int b = 4,
+    int boot = 1,
+    int threads = 8,
     unsigned int seed = 42,
     int parallel_level = 0,
     int dist_metric = 2,
@@ -197,7 +239,7 @@ std::vector<std::vector<double>> MultispatialCCM(
   threads_sizet = std::min(static_cast<size_t>(std::thread::hardware_concurrency()), threads_sizet);
 
   // Transform to ensure no size exceeds max library size
-  int max_lib_size = x.size();
+  int max_lib_size = lib.size();
 
   std::vector<int> unique_lib_sizes(lib_sizes.begin(), lib_sizes.end());
   std::transform(unique_lib_sizes.begin(), unique_lib_sizes.end(), unique_lib_sizes.begin(),
@@ -222,9 +264,10 @@ std::vector<std::vector<double>> MultispatialCCM(
         local_results[i] = SimplexPredictionBoot(
           x,
           y,
+          unique_lib_sizes[i],
+          lib,
           E,
           tau,
-          unique_lib_sizes[i],
           b,
           boot,
           threads_sizet,
@@ -239,9 +282,10 @@ std::vector<std::vector<double>> MultispatialCCM(
         local_results[i] = SimplexPredictionBoot(
           x,
           y,
+          unique_lib_sizes[i],
+          lib,
           E,
           tau,
-          unique_lib_sizes[i],
           b,
           boot,
           threads_sizet,
@@ -260,9 +304,10 @@ std::vector<std::vector<double>> MultispatialCCM(
         local_results[i] = SimplexPredictionBoot(
           x,
           y,
+          lib_size,
+          lib,
           E,
           tau,
-          lib_size,
           b,
           boot,
           threads_sizet,
@@ -278,9 +323,10 @@ std::vector<std::vector<double>> MultispatialCCM(
         local_results[i] = SimplexPredictionBoot(
           x,
           y,
+          lib_size,
+          lib,
           E,
           tau,
-          lib_size,
           b,
           boot,
           threads_sizet,
