@@ -9,11 +9,9 @@
 #include "Embed.h"
 #include "SimplexProjection.h"
 #include "SMap.h"
-#include "tEDMDataStruct.h"
+#include "DataStruct.h"
 #include <RcppThread.h>
 
-// [[Rcpp::depends(RcppThread)]]
-
 /**
  * @brief Computes the partial correlation between the target variable and its simplex projection,
  *        incorporating control variables using a time-delay embedding approach.
@@ -334,6 +332,7 @@ std::vector<PartialCorRes> PCMSingle(
  * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * - dist_metric: Distance metric selector (1: Manhattan, 2: Euclidean).
  * - dist_average: Whether to average distance by the number of valid vector components.
+ * - single_sig: Whether to estimate significance and confidence intervals using a single rho value.
  * - progressbar: Boolean flag indicating whether to display a progress bar during computation.
  *
  * Returns:
@@ -341,12 +340,12 @@ std::vector<PartialCorRes> PCMSingle(
  *      - The library size.
  *      - The mean pearson cross-mapping correlation.
  *      - The statistical significance of the pearson correlation.
- *      - The upper bound of the pearson correlation confidence interval.
  *      - The lower bound of the pearson correlation confidence interval.
+ *      - The upper bound of the pearson correlation confidence interval.
  *      - The mean partial cross-mapping partial correlation.
  *      - The statistical significance of the partial correlation.
- *      - The upper bound of the partial correlation confidence interval.
  *      - The lower bound of the partial correlation confidence interval.
+ *      - The upper bound of the partial correlation confidence interval.
  */
 std::vector<std::vector<double>> PCM(
     const std::vector<double>& x,                       // Time series to cross map from
@@ -365,6 +364,7 @@ std::vector<std::vector<double>> PCM(
     bool cumulate,                                      // Whether to cumulate the partial correlations
     int dist_metric,                                    // Distance metric selector (1: Manhattan, 2: Euclidean)
     bool dist_average,                                  // Whether to average distance by the number of valid vector components
+    bool single_sig,                                    // Whether to estimate significance and confidence intervals using a single rho value
     bool progressbar                                    // Whether to print the progress bar
 ) {
   // If b is not provided correctly, default it to E + 2
@@ -529,81 +529,82 @@ std::vector<std::vector<double>> PCM(
     grouped_results[result.first].emplace_back(result.second, result.third);
   }
 
-  // // Previous implementation calculated significance and confidence intervals using the mean of rho vector only.
-  // // This approach is now deprecated and kept here for comparison purposes.
-  // std::vector<std::vector<double>> final_results;
-  // // Compute the mean of second and third values for each group
-  // for (const auto& group : grouped_results) {
-  //   std::vector<double> second_values, third_values;
-  //
-  //   for (const auto& val : group.second) {
-  //     second_values.push_back(val.first);
-  //     third_values.push_back(val.second);
-  //   }
-  //
-  //   double mean_second = CppMean(second_values, true);
-  //   double mean_third = CppMean(third_values, true);
-  //
-  //   final_results.push_back({static_cast<double>(group.first), mean_second, mean_third});
-  // }
-  //
-  // // Compute significance and confidence intervals for each result
-  // for (size_t i = 0; i < final_results.size(); ++i) {
-  //   double rho_second = final_results[i][1];
-  //   double rho_third = final_results[i][2];
-  //
-  //   // Compute significance and confidence interval for second value
-  //   double significance_second = CppCorSignificance(rho_second, n);
-  //   std::vector<double> confidence_interval_second = CppCorConfidence(rho_second, n);
-  //
-  //   // Compute significance and confidence interval for third value
-  //   double significance_third = CppCorSignificance(rho_third, n, n_confounds);
-  //   std::vector<double> confidence_interval_third = CppCorConfidence(rho_third, n, n_confounds);
-  //
-  //   // Append computed statistical values to the result
-  //   final_results[i].push_back(significance_second);
-  //   final_results[i].push_back(confidence_interval_second[0]);
-  //   final_results[i].push_back(confidence_interval_second[1]);
-  //
-  //   final_results[i].push_back(significance_third);
-  //   final_results[i].push_back(confidence_interval_third[0]);
-  //   final_results[i].push_back(confidence_interval_third[1]);
-  // }
-
-  // For each group, compute the mean of second and third values and calculate significance and confidence intervals using the original vectors
   std::vector<std::vector<double>> final_results;
-  for (const auto& group : grouped_results) {
-    std::vector<double> second_values, third_values;
 
-    // Collect all second and third values from current group
-    for (const auto& val : group.second) {
-      second_values.push_back(val.first);
-      third_values.push_back(val.second);
+  if (single_sig) {
+    // Calculate significance and confidence intervals using the mean of rho vector only.
+    for (const auto& group : grouped_results) {
+      // Compute the mean of second and third values for each group
+      std::vector<double> second_values, third_values;
+
+      for (const auto& val : group.second) {
+        second_values.push_back(val.first);
+        third_values.push_back(val.second);
+      }
+
+      double mean_second = CppMean(second_values, true);
+      double mean_third = CppMean(third_values, true);
+
+      final_results.push_back({static_cast<double>(group.first), mean_second, mean_third});
     }
 
-    // Compute mean values for reporting
-    double mean_second = CppMean(second_values, true);
-    double mean_third = CppMean(third_values, true);
+    // Compute significance and confidence intervals for each result
+    for (size_t i = 0; i < final_results.size(); ++i) {
+      double rho_second = final_results[i][1];
+      double rho_third = final_results[i][2];
 
-    // Compute significance and confidence intervals using the full vector of values (not just mean)
-    double significance_second = CppMeanCorSignificance(second_values, n);
-    std::vector<double> confidence_interval_second = CppMeanCorConfidence(second_values, n);
+      // Compute significance and confidence interval for second value
+      double significance_second = CppCorSignificance(rho_second, n);
+      std::vector<double> confidence_interval_second = CppCorConfidence(rho_second, n);
 
-    double significance_third = CppMeanCorSignificance(third_values, n, n_confounds);
-    std::vector<double> confidence_interval_third = CppMeanCorConfidence(third_values, n, n_confounds);
+      // Compute significance and confidence interval for third value
+      double significance_third = CppCorSignificance(rho_third, n, n_confounds);
+      std::vector<double> confidence_interval_third = CppCorConfidence(rho_third, n, n_confounds);
 
-    // Store group ID, mean values, and corresponding statistical results
-    final_results.push_back({
-      static_cast<double>(group.first),
-      mean_second,
-      mean_third,
-      significance_second,
-      confidence_interval_second[0],
-      confidence_interval_second[1],
-      significance_third,
-      confidence_interval_third[0],
-      confidence_interval_third[1]
-    });
+      // Append computed statistical values to the result
+      final_results[i].push_back(significance_second);
+      final_results[i].push_back(confidence_interval_second[0]);
+      final_results[i].push_back(confidence_interval_second[1]);
+
+      final_results[i].push_back(significance_third);
+      final_results[i].push_back(confidence_interval_third[0]);
+      final_results[i].push_back(confidence_interval_third[1]);
+    }
+  } else {
+    // For each group, compute the mean of second and third values and calculate significance and confidence intervals using the original vectors
+    for (const auto& group : grouped_results) {
+      std::vector<double> second_values, third_values;
+
+      // Collect all second and third values from current group
+      for (const auto& val : group.second) {
+        second_values.push_back(val.first);
+        third_values.push_back(val.second);
+      }
+
+      // Compute mean values for reporting
+      double mean_second = CppMean(second_values, true);
+      double mean_third = CppMean(third_values, true);
+
+      // Compute significance and confidence intervals using the full vector of values (not just mean)
+      double significance_second = CppMeanCorSignificance(second_values, n);
+      std::vector<double> confidence_interval_second = CppMeanCorConfidence(second_values, n);
+
+      double significance_third = CppMeanCorSignificance(third_values, n, n_confounds);
+      std::vector<double> confidence_interval_third = CppMeanCorConfidence(third_values, n, n_confounds);
+
+      // Store group ID, mean values, and corresponding statistical results
+      final_results.push_back({
+        static_cast<double>(group.first),
+        mean_second,
+        mean_third,
+        significance_second,
+        confidence_interval_second[0],
+        confidence_interval_second[1],
+        significance_third,
+        confidence_interval_third[0],
+        confidence_interval_third[1]
+      });
+    }
   }
 
   return final_results;
